@@ -1,30 +1,16 @@
-import json
 import re
 from collections import defaultdict
 import copy
-import os
 
 from typing import DefaultDict, Dict, List, NamedTuple, Set, Tuple, Optional, Literal, TypedDict
+from yajwiz.boqwiz import BoqwizEntry, load_dictionary
 
 from .tables import SUFFIX_TYPES, UNIVERSAL_FEATURES, XPOS_TO_UPOS, PREFIX_TABLE, Person, Number
 
 Xpos = Literal["VS", "VT", "VI", "VA", "V?", "NL", "NB", "PRON", "NUM", "N", "ADV", "EXCL", "CONJ", "QUES", "UNK"]
 
-class BoqwizEntry(TypedDict):
-    entry_name: str
-    part_of_speech: str
-
-def _parse_boqwiz_pos(pos: str) -> Set[str]:
-    colon = pos.split(":")
-    if len(colon) == 1:
-        return set(colon)
-    
-    [tpos, tags] = colon
-    comma = tags.split(",")
-    return set([tpos]+comma)
-
 def _get_xpos(entry: BoqwizEntry) -> Xpos:
-    pos2 = _parse_boqwiz_pos(entry["part_of_speech"])
+    pos2 = entry.tags
     if "v" in pos2 and "is" in pos2:
         return "VS"
     
@@ -46,10 +32,10 @@ def _get_xpos(entry: BoqwizEntry) -> Xpos:
     elif pos2 & {"n"} and pos2 & {"body"}:
         return "NB"
     
-    elif pos2 & {"n"} and ("pro" in pos2 or entry["entry_name"] in {"'Iv", "nuq", "jIH"}):
+    elif pos2 & {"n"} and ("pro" in pos2 or entry.name in {"'Iv", "nuq", "jIH"}):
         return "PRON"
     
-    elif pos2 & {"n"} and entry["entry_name"] in {"wa'", "cha'", "wej", "loS", "vagh", "jav", "Soch", "chorgh", "Hut"}:
+    elif pos2 & {"n"} and entry.name in {"wa'", "cha'", "wej", "loS", "vagh", "jav", "Soch", "chorgh", "Hut"}:
         return "NUM"
     
     elif pos2 & {"n"}:
@@ -70,10 +56,7 @@ def _get_xpos(entry: BoqwizEntry) -> Xpos:
     else:
         return "UNK"
 
-SCRIPT_DIR = os.path.dirname(os.path.realpath(__file__))
-
-with open(os.path.join(SCRIPT_DIR, "data.json"), "r") as f:
-    data = json.load(f)
+dictionary = load_dictionary()
 
 VERBS: List[str] = []
 STATIVE_VERBS: List[str] = ["lo'laH", "lo'laHbe'"]
@@ -85,20 +68,19 @@ DERIV_NOUNS: List[str] = []
 
 ALL_WORDS: Set[str] = set()
 WORD_INDEX: DefaultDict[str, List[BoqwizEntry]] = defaultdict(lambda: [])
-XPOS_INDEX: DefaultDict[str, Set[BoqwizEntry]] = defaultdict(set)
+XPOS_INDEX: DefaultDict[str, Set[str]] = defaultdict(set)
 
-for boqwiz_id in data["qawHaq"]:
-    entry = data["qawHaq"][boqwiz_id]
-    entry["id"] = boqwiz_id
-    word = entry["entry_name"]
-    pos = _parse_boqwiz_pos(entry["part_of_speech"])
+for boqwiz_id in dictionary.entries:
+    entry = dictionary.entries[boqwiz_id]
+    word = entry.name
+    pos = entry.tags
     if "hyp" in pos:
         continue
 
-    very_bad = " " in word or "pref" in pos or "suff" in pos
-    good = not (very_bad or "deriv" in pos or word in DERIV_NOUNS+DERIV_VERBS+DERIV_STATIVE_VERBS)
+    very_bad = "pref" in pos or "suff" in pos
 
     if "v" in pos:
+        good = not very_bad and word not in DERIV_VERBS and word not in DERIV_STATIVE_VERBS
         if good:
             VERBS.append(word)
         
@@ -114,7 +96,7 @@ for boqwiz_id in data["qawHaq"]:
             DERIV_STATIVE_VERBS.append(word)
     
     elif "n" in pos:
-        if good:
+        if not very_bad and word not in DERIV_NOUNS:
             NOUNS.append(word)
         
         elif not very_bad:
@@ -327,13 +309,13 @@ def _analyze_word_with_pos(ans: List[Analysis], start_pos: str, regex: re.Patter
                 objs = []
                 for entry in WORD_INDEX[part + ":" + pos]:
                     new_obj = copy.deepcopy(obj)
-                    new_obj["PARTS"].append(entry["id"])
+                    new_obj["PARTS"].append(entry.id)
                     if i == lemma_idx:
                         #new_obj["BOQWIZ"] = entry
 
                         new_obj["XPOS"] = _get_xpos(entry)
-                        new_obj["BOQWIZ_POS"] = entry["part_of_speech"]
-                        new_obj["BOQWIZ_ID"] = entry["id"]
+                        new_obj["BOQWIZ_POS"] = entry.part_of_speech
+                        new_obj["BOQWIZ_ID"] = entry.id
 
                         if not lemma_pred(entry):
                             continue
@@ -392,21 +374,21 @@ def analyze(word: str, include_syntactical_info=False) -> List[Analysis]:
     _analyze_word_with_pos(ans, "n", NOUN_REGEX, 0, word)
     _analyze_word_with_pos(ans, "n", NUMBER_REGEX, 0, word)
     if not ans or len(ans[0]["PARTS"]) > 1:
-        _analyze_word_with_pos(ans, "n", PRONOUN_VERB_REGEX, 0, word, infl_pos="v", lemma_pred=lambda e: "pro" in e["part_of_speech"])
+        _analyze_word_with_pos(ans, "n", PRONOUN_VERB_REGEX, 0, word, infl_pos="v", lemma_pred=lambda e: "pro" in e.tags)
 
     _analyze_word_with_pos(ans, "v", VERB_REGEX, 1, word)
-    _analyze_word_with_pos(ans, "v", STATIVE_VERB_REGEX, 0, word, lemma_pred=lambda e: "is" in e["part_of_speech"])
+    _analyze_word_with_pos(ans, "v", STATIVE_VERB_REGEX, 0, word, lemma_pred=lambda e: "is" in e.tags)
 
     if word + ":other" in WORD_INDEX:
         for entry in WORD_INDEX[word + ":other"]:
             ans.append({
                 "WORD": word,
-                "LEMMA": entry["entry_name"],
+                "LEMMA": entry.name,
                 "POS": "OTHER",
                 "XPOS": _get_xpos(entry),
-                "BOQWIZ_POS": entry["part_of_speech"],
-                "BOQWIZ_ID": entry["id"],
-                "PARTS": [ entry["id"] ],
+                "BOQWIZ_POS": entry.part_of_speech,
+                "BOQWIZ_ID": entry.id,
+                "PARTS": [ entry.id ],
             })
     
     for analysis in ans:
@@ -564,6 +546,7 @@ def tokenize(sentence: str) -> List[Tuple[TokenType, str]]:
 class ProofreaderError(NamedTuple):
     message: str
     location: int
+    end_location: int
 
 def get_errors(sentence: str) -> List[ProofreaderError]:
     errors: List[ProofreaderError] = []
@@ -577,19 +560,21 @@ def get_errors(sentence: str) -> List[ProofreaderError]:
     
     for i in range(len(tokens)):
         pos, token1_type, token1_text = tokens[i]
+        end_pos = pos + len(token1_text)
         if token1_type == "WORD":
             token1 = analyze(token1_text)
             if not token1:
-                errors.append(ProofreaderError(f"UNKNOWN WORD {token1_text}", pos))
+                errors.append(ProofreaderError(f"UNKNOWN WORD {token1_text}", pos, end_pos))
                 continue
 
             if all(token.get("UNGRAMMATICAL", None) for token in token1):
-                errors.append(ProofreaderError(token1[0]["UNGRAMMATICAL"], pos))
+                errors.append(ProofreaderError(token1[0]["UNGRAMMATICAL"], pos, end_pos))
 
             if i > len(tokens)-2:
                 continue
 
-            _, token2_type, token2_text = tokens[i+1]
+            token2_pos, token2_type, token2_text = tokens[i+1]
+            token2_end_pos = token2_pos + len(token2_text)
 
             if not token2_type or token2_type != "WORD":
                 continue
@@ -600,9 +585,10 @@ def get_errors(sentence: str) -> List[ProofreaderError]:
 
             if token1_text == "'e'":
                 if all("V7" in token.get("SUFFIX", {}) for token in token2):
-                    errors.append(ProofreaderError("ASPECT SUFFIX IN COMPLEX SENTENCE", pos+1))
+                    errors.append(ProofreaderError("ASPECT SUFFIX IN COMPLEX SENTENCE", pos, token2_end_pos))
 
-                _, token3_type, token3_text = tokens[i+2] if i <= len(tokens)-3 else (0, None, "")
+                token3_pos, token3_type, token3_text = tokens[i+2] if i <= len(tokens)-3 else (0, None, "")
+                token3_end_pos = token3_pos + len(token3_text)
 
                 if token3_type and token3_type == "WORD":
                     token3 = analyze(token3_text, True)
@@ -611,10 +597,10 @@ def get_errors(sentence: str) -> List[ProofreaderError]:
                     token3 = []
 
                 if token2_text in {"neH", "neHHa'", "je"} and token3 and all("V7" in token.get("SUFFIX", {}) for token in token3):
-                    errors.append(ProofreaderError("ASPECT SUFFIX IN COMPLEX SENTENCE", pos+2))
+                    errors.append(ProofreaderError("ASPECT SUFFIX IN COMPLEX SENTENCE", pos, token3_end_pos))
 
                 if all(token["BOQWIZ_ID"] == "neH:v" for token in token2) or token2_text == "neH" and not any(3 in token.get("SYNTAX_INFO", {}).get("OBJECT_PERSON", set()) for token in token3):
-                    errors.append(ProofreaderError("'e' WITH neH", pos))
+                    errors.append(ProofreaderError("'e' WITH neH", pos, token2_end_pos))
 
     return errors
 
